@@ -6,7 +6,7 @@ use POSIX;
 use List::Util;
 use Data::Dumper qw(Dumper);
 use Exporter qw(import);
-our @EXPORT_OK = qw(
+our @EXPORT_LIST = qw(
     incr        reduces   flatten
     drop_right  drop      take_right  take
     assoc       maps      decr        chain
@@ -20,17 +20,22 @@ our @EXPORT_OK = qw(
     second      range     pops        pushes
     shifts      unshifts  once
 );
-
-our $VERSION = '0.38';
-
+our @EXPORT_OK                 = @EXPORT_LIST;
+our $VERSION                   = '0.39';
 use constant ARG_PLACE_HOLDER => {};
 
+_wrap_to_use_partials(
+    grep { $_ !~ /flow|flow_right|partial|chain|range/ } @EXPORT_LIST
+);
+
 # -----------------------------------------------------------------------------#
 
 sub __ { ARG_PLACE_HOLDER };
 
-sub _has_placeholders {
-    foreach my $arg (@_) {
+sub _contains_placeholders {
+    my $arguments = [@_];
+
+    foreach my $arg (@{ $arguments }) {
         if (ref($arg) and ref(__) and $arg == __) {
             return 1;
         }
@@ -38,6 +43,34 @@ sub _has_placeholders {
     return 0;
 }
 
+sub _wrap {
+    no warnings 'redefine';
+    no strict 'refs';
+    my ($methodName, $decorator) = @_;
+    my $module                    = caller;
+    my $fullName                  = "$module"."::"."$methodName";
+    my $wrappedSub                = *{"$fullName"}{CODE};
+
+    *{"$fullName"} = List::Util::reduce {
+       my ($wrappedSub, $decor) = ($a, $b);
+       return $decor->($wrappedSub);
+    } ($wrappedSub, $decorator);
+}
+
+sub _can_use_placeholders {
+    my $func = shift;
+
+    return sub {
+        return _contains_placeholders(@_) ? partial($func, @_) : $func->(@_);
+    }
+}
+
+sub _wrap_to_use_partials {
+    foreach my $func_name (@_) {
+        _wrap($func_name, \&_can_use_placeholders);
+    }
+}
+
 sub noop { return undef }
 
 sub identity {
@@ -456,7 +489,6 @@ sub maps {
     my $func = shift;
     my $coll = shift;
 
-
     my $idx = 0;
 
     my @vals = map {
@@ -524,7 +556,7 @@ sub _fill_holders {
     for (my $idx = 0; $idx < $old_args_len; $idx++) {
         my $arg = shift @{ $old_args };
 
-        if (eql($arg, __)) {
+        if(_contains_placeholders($arg)) {
             push @{ $filled_args }, (shift @{ $new_args });
         } else {
             push @{ $filled_args }, $arg;
@@ -602,6 +634,8 @@ sub spread {
     return split('', $coll);
 }
 
+# ------------------------------------------------------------------------------
+
 =head1 NAME
 
 Sub::Fp - A Clojure / Python Toolz / Lodash inspired Functional Utility Library
